/*
 * Abstract unbalanced binary tree.
 *
 * This software is distributed under the terms of GNU
 * Public License version 2 or later versions.
 *
 */

#ifndef _BIN_TREE_H
#define _BIN_TREE_H

typedef int boolean_t;

#ifndef FALSE
  #define FALSE	(0)
#endif
#ifndef TRUE
  #define TRUE	(!FALSE)
#endif

typedef enum {
	BIN_TREE_PREORDER,
	BIN_TREE_INORDER,
	BIN_TREE_POSTORDER
} bin_tree_order_t;

struct bin_tree;	/* Abstract type to shut up gcc. */
struct bin_tree_node;

typedef int (*bin_tree_compare_t)(void *a, void *b);
typedef int (*bin_tree_callback_t)(struct bin_tree_node *np);

struct bin_tree_node {
	struct bin_tree_node	*parent;
	struct bin_tree_node	*left;
	struct bin_tree_node	*right;
	void			*key;	/* pointer to node's key */
	void			*data;	/* pointer to node's data */
};

/* Create new abstract tree. */
struct bin_tree *bin_tree_new(bin_tree_compare_t cmp);

/* Free all resouces occupied by the tree and it's nodes. */
void bin_tree_free(struct bin_tree **tree);

/* Add node to the tree. */
boolean_t bin_tree_add(struct bin_tree *tree, struct bin_tree_node *np);

/* Add data to the tree. */
struct bin_tree_node *bin_tree_add_data(struct bin_tree *tree, void *key, void *data, boolean_t replace);

/* Delete node from the tree. */
boolean_t bin_tree_delete(struct bin_tree *tree, struct bin_tree_node *np);

/* Delete data from the tree. */
boolean_t bin_tree_delete_data(struct bin_tree *tree, void *key);

/* Find node by it's key. */
struct bin_tree_node *bin_tree_find(struct bin_tree *tree, void *key);

/* Traverse the tree using callback function. */
void bin_tree_traverse(struct bin_tree *tree, bin_tree_callback_t callback, bin_tree_order_t order);

/* Get pointer to next or previous node of the tree. */
struct bin_tree_node *bin_tree_skip_to(struct bin_tree *tree, struct bin_tree_node *np, int which);

/* Get pointer to next node. */
#define bin_tree_next(tree, np) binary_tree_skip_to(tree, np, 0)

/* Get pointer to previous node. */
#define bin_tree_prev(tree, np) binary_tree_skip_to(tree, np, 1)

#endif

